using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CrystalDecisions.Enterprise;
using CrystalDecisions.Enterprise.Desktop;
using CrystalDecisions.Shared;
/*
 * Sample program showing how to access the User and Groups classes in the
 * BusinessObjects XI SDK.
 */
namespace BusinessObjects_User_Info_Tutorial
{
  public partial class User_Info_Tutorial_Form : Form
  {
    private EnterpriseSession _BOESession = null;
    private EnterpriseService _BOEService = null;
    private bool _initialized = false;
    private const string C_NUMBERS = "0123456789";
    private InfoStore _BOEInfoStore = null;

    public User_Info_Tutorial_Form()
    {
      InitializeComponent();
    }

    private void User_Info_Tutorial_Form_FormClosing(object sender, FormClosingEventArgs e)
    {
      _BOEService = null;
      if (_initialized)
      {
        /*The SDK is a wrapper for COM objects.  In order to manage resources properly
          ALWAYS explicitly dispose of your objects when you're done with them.*/
        if (_BOEInfoStore != null)
          _BOEInfoStore.Dispose();
        if (_BOESession != null)
        {
          _BOESession.Logoff();
          _BOESession.Dispose();
        }
      }
    }

    private void cmsBtn_Click(object sender, EventArgs e)
    {
      if ((cmsUser.Text == string.Empty) || (cmsName.Text == string.Empty))
      {
        MessageBox.Show("Unable to log in to BOE - User ID or CMS Name not set",
                        "Login Failure", MessageBoxButtons.OK);
      }
      if (_initialized)
      {
        //if we're already logged in to a CMS, log off first
        _BOESession.Logoff();
        _BOEService = null;
        _initialized = false;
        cmsStatus.Text = "NOT Logged in to CMS!";
        cmsStatus.Refresh();
      }
      //log in to CMS
      using (SessionMgr sessionMgr = new SessionMgr())
      {
        try
        {
          cmsStatus.Text = "Logging in to CMS...";
          cmsStatus.Refresh();
          //Login
          _BOESession = sessionMgr.Logon(cmsUser.Text, cmsPW.Text, cmsName.Text, "secEnterprise");
          //Get the InfoStore service
          _BOEService = _BOESession.GetService("InfoStore");
          //Cast from an EnterpriseService to an InfoStore so we can use it.
          _BOEInfoStore = (InfoStore)_BOEService;
          _initialized = true;
          cmsStatus.Text = "Logged in to CMS " + cmsName.Text + ".";
          cmsStatus.Refresh();
          tutorialTabs.Enabled = true;
        }
        catch (Exception ex)
        {
          MessageBox.Show("Unable to log in to BOE - " + ex.Message);
        }
      }
    }

    #region Users
    private void userListBtn_Click(object sender, EventArgs e)
    {
      userStatus.Text = "Loading Users from CMS. Please Wait...";
      userStatus.Refresh();
      //Clear out the existing tree and disable updates while we're loading data.
      userTree.Nodes.Clear();
      userTree.BeginUpdate();
      try
      {
        //get BOE user list
        string query = "SELECT SI_NAME, SI_USERFULLNAME, SI_ID, SI_USERGROUPS, " +
                       "  SI_NAMEDUSER, SI_CHANGEPASSWORD, SI_PASSWORDEXPIRE, " +
                       "  SI_EMAIL_ADDRESS " +
                       "From CI_SYSTEMOBJECTS " +
                       "WHERE SI_KIND='User' and SI_NAME != 'Administrator' and SI_NAME != 'Guest'" +
                       "ORDER BY SI_NAME";

        using (InfoObjects users = _BOEInfoStore.Query(query))
        {
          User userData = null;
          TreeNode newNode = null;
          try
          {
            //foreach doesn't work with an InfoObjects collection so we have to use a regular
            //for loop.  InfoObjects is 1 indexed instead of .NETS usual 0 index.
            for (int i = 1; i <= users.Count; i++)
            {
              userData = (User)users[i];
              //The user name is in the FullName property of the user object
              //The user ID is in the Title property of the user object
              //users.count
              newNode = new TreeNode(userData.FullName + " - " + userData.Title);
              newNode.Tag = loadUserInfo(userData);
              userTree.Nodes.Add(loadUserGroupNodes(newNode, userData.Groups));
            }
          }
          finally
          {
            //Explicitly dispose of the userData object to clean up memory and release the COM connection.
            if (userData != null)
              userData.Dispose();
            userStatus.Text = string.Empty;
            userStatus.Refresh();
          }
        } //using users
      }
      catch (Exception ex)
      {
        MessageBox.Show("Unable to load user list.  Err=" + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        userTree.EndUpdate();
      }
    }

    private List<string> loadUserInfo(User userData)
    {
      //get some basic info about the user so we can display it.
      List<string> result = new List<string>();
      result.Add("Internal ID: " + userData.ID.ToString());
      result.Add("User Can Change PW? " + userData.AllowChangePassword.ToString());
      result.Add("User Password Expires? " + userData.PasswordExpires.ToString());
      result.Add("User Type: " + userData.Connection.ToString());
      result.Add("Email Address: " + userData.EmailAddress);
      return result;
    }

    private TreeNode loadUserGroupNodes(TreeNode node, Groups userGroups)
    {
      StringBuilder sb = new StringBuilder();
      //build the list of group id's for our query
      foreach (int groupID in userGroups)
      {
        sb.Append(groupID.ToString() + ", ");
      }
      if (sb.Length > 0)
      {
        string query = "Select SI_NAME, SI_DESCRIPTION From CI_SYSTEMOBJECTS where SI_ID in (" +
           sb.ToString().Substring(0, sb.Length - 2) + ") Order By SI_NAME"; //strip off trailing ", "
        TreeNode newNode;
        using (InfoObjects groupInfo = _BOEInfoStore.Query(query))
        {
          for (int i = 1; i <= groupInfo.Count; i++)
          {
            newNode = new TreeNode(groupInfo[i].Title + " - " + groupInfo[i].Description);
            newNode.Tag = string.Empty;
            node.Nodes.Add(newNode);
          }
        }
      }
      return node;
    }

    private void userTree_AfterSelect(object sender, TreeViewEventArgs e)
    {
      //load userInfo into list
      this.Cursor = Cursors.WaitCursor;
      userInfolb.Items.Clear();
      userInfolb.BeginUpdate();
      try
      {
        if (userTree.SelectedNode.Nodes.Count > 0)
        {
          List<string> userInfo = (List<string>)userTree.SelectedNode.Tag;
          foreach (string info in userInfo)
            userInfolb.Items.Add(info);
        }
      }
      finally
      {
        userInfolb.EndUpdate();
        this.Cursor = Cursors.Default;
      }
    }

    #endregion Users

    #region Groups

    private void groupListBtn_Click(object sender, EventArgs e)
    {
      groupStatus.Text = "Loading Groups from CMS.  Please Wait...";
      groupStatus.Refresh();
      this.Cursor = Cursors.WaitCursor;
      groupTree.Nodes.Clear();
      groupTree.BeginUpdate();

      //Clear out the existing table and disable grid updates while we're loading data.

      try
      {
        //get BOE user group list
        string query = "SELECT SI_ID, SI_NAME, SI_DESCRIPTION, SI_GROUP_MEMBERS " +
                       "FROM CI_SYSTEMOBJECTS " +
                       "WHERE SI_KIND ='UserGroup' " +
                       "ORDER BY SI_NAME";
        using (InfoObjects groups = _BOEInfoStore.Query(query))
        {
          try
          {
            TreeNode newNode;
            for (int i = 1; i <= groups.Count; i++)
            {
              //if we use "using" we don't have to explicitly dispose of the objects - 
              //it's done for us automatically when the object loses scope
              using (UserGroup groupData = (UserGroup)groups[i])
              {
                newNode = new TreeNode(groupData.Title);
                newNode.Tag = groupData.Description;
                newNode = loadGroupUserNodes(newNode, groupData.Users);
                groupTree.Nodes.Add(newNode);
              }
              
            } //groups.count
          }
          finally
          {
            userStatus.Text = string.Empty;
            userStatus.Refresh();
          }
        } //using users
      }
      catch (Exception ex)
      {
        MessageBox.Show("Unable to load group list.  Err=" + ex.Message, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        groupTree.EndUpdate();
        this.Cursor = Cursors.Default;
      }
    }

    private TreeNode loadGroupUserNodes(TreeNode node, GroupAssociates userList)
    {
      StringBuilder sb = new StringBuilder();
      //build the list of group id's for our query
      foreach (int userID in userList)
      {
        sb.Append(userID.ToString() + ", ");
      }
      if (sb.Length > 0)
      {
        string query = "Select SI_NAME, SI_USERFULLNAME From CI_SYSTEMOBJECTS where SI_ID in (" +
           sb.ToString().Substring(0, sb.Length - 2) + ") Order By SI_NAME"; //strip off trailing ", "
        TreeNode newNode;
        User user;
        using (InfoObjects groupInfo = _BOEInfoStore.Query(query))
        {
          for (int i = 1; i <= groupInfo.Count; i++)
          {
            //in-line casting doesn't always work, so we have to have an actual "User" object to 
            //get to the FullName property
            user = (User)groupInfo[i];
            newNode = new TreeNode(user.Title + " - " + user.FullName);
            newNode.Tag = string.Empty;
            node.Nodes.Add(newNode);
          }
        }
      }
      return node;
    }

    private void groupTree_AfterSelect(object sender, TreeViewEventArgs e)
    {
      //load userInfo into list
      this.Cursor = Cursors.WaitCursor;
      groupInfolb.Items.Clear();
      groupInfolb.Items.Add(groupTree.SelectedNode.Tag.ToString());
      this.Cursor = Cursors.Default;
    }

    #endregion Groups

    #region AddNew

    private void tabNew_Enter(object sender, EventArgs e)
    {
      cbConnect.SelectedIndex = 0;
    }

    private void btnAddGroup_Click(object sender, EventArgs e)
    {
      this.Cursor = Cursors.WaitCursor;
      try
      {
        PluginManager pm = _BOEInfoStore.PluginManager;
        PluginInfo pi = pm.GetPluginInfo("CrystalEnterprise.UserGroup");
        InfoObjects userObjs = _BOEInfoStore.NewInfoObjectCollection();
        userObjs.Add(pi);
        UserGroup newGroup = (UserGroup)userObjs[1];
        newGroup.Title = txtGrpName.Text;
        newGroup.Description = txtGrpDesc.Text;
        _BOEInfoStore.Commit(userObjs);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        this.Cursor = Cursors.Default;
      }

    }

    private void btnAddUser_Click(object sender, EventArgs e)
    {
      this.Cursor = Cursors.WaitCursor;
      try
      {
        PluginManager pm = _BOEInfoStore.PluginManager;
        PluginInfo pi = pm.GetPluginInfo("CrystalEnterprise.User");
        InfoObjects userObjs = _BOEInfoStore.NewInfoObjectCollection();
        userObjs.Add(pi);
        User newUser = (User)userObjs[1];
        newUser.Title = txtUserID.Text;
        newUser.FullName = txtUserName.Text;
        newUser.Connection = CeConnectionType.ceConnectionNamed;
        newUser.PasswordExpires = chkExpires.Checked;
        newUser.ChangePasswordAtNextLogon = chkChangeNext.Checked;
        newUser.AllowChangePassword = chkCanChange.Checked;
        newUser.NewPassword = txtPassword.Text;
        newUser.EmailAddress = txtEmail.Text;
        _BOEInfoStore.Commit(userObjs);
      }
      catch (Exception ex)
      {
      }
      finally
      {
        this.Cursor = Cursors.Default;
      }

    }

    #endregion AddNew

  }
}